@@ -1,10 +1,10 @@
-/* $Id: strformat.cpp 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: strformat.cpp 135976 2020-02-04 10:35:17Z bird $ */
 /** @file
  * IPRT - String Formatter.
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,17 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Defined Constants                                                          *
-*******************************************************************************/
-#define ISDIGIT(c) ((c) >= '0' && (c) <= '9')
-/*#define MAX(a, b)  ((a) >= (b) ? (a) : (b))
-#define MIN(a, b)  ((a) < (b) ? (a) : (b)) */
-
-
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP RTLOGGROUP_STRING
 #include <iprt/string.h>
 #include "internal/iprt.h"
@@ -43,32 +35,15 @@
 #include <iprt/assert.h>
 #ifdef IN_RING3
 # include <iprt/alloc.h>
-# include <iprt/err.h>
+# include <iprt/errcore.h>
 # include <iprt/uni.h>
+# include <iprt/utf16.h>
 #endif
+#include <iprt/ctype.h>
 #include <iprt/string.h>
 #include <iprt/stdarg.h>
 #include "internal/string.h"
 
-/* Wrappers for converting to iprt facilities. */
-#define SSToDS(ptr)     ptr
-#define kASSERT         Assert
-#define KENDIAN_LITTLE  1
-#define KENDIAN         KENDIAN_LITTLE
-#define KSIZE           size_t
-typedef struct
-{
-    uint32_t    ulLo;
-    uint32_t    ulHi;
-} KSIZE64;
-
-
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
-static unsigned _strnlen(const char *psz, unsigned cchMax);
-static unsigned _strnlenUtf16(PCRTUTF16 pwsz, unsigned cchMax);
-static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, signed int cchWidth, signed int cchPrecision, unsigned int fFlags);
 
 
 /**
@@ -139,49 +114,29 @@ static unsigned _strnlenUni(PCRTUNICP pusz, unsigned cchMax)
 /**
  * Formats an integer number according to the parameters.
  *
- * @returns   Length of the formatted number.
- * @param     psz            Pointer to output string buffer of sufficient size.
- * @param     u64Value       Value to format.
+ * @returns   Length of the number.
+ * @param     psz            Pointer to output string.
+ * @param     u64Value       Value.
  * @param     uiBase         Number representation base.
- * @param     cchWidth       Width.
+ * @param     cchWidth       Width
  * @param     cchPrecision   Precision.
  * @param     fFlags         Flags (NTFS_*).
  */
 RTDECL(int) RTStrFormatNumber(char *psz, uint64_t u64Value, unsigned int uiBase, signed int cchWidth, signed int cchPrecision,
                               unsigned int fFlags)
-{
-    return rtStrFormatNumber(psz, *(KSIZE64 *)(void *)&u64Value, uiBase, cchWidth, cchPrecision, fFlags);
-}
-RT_EXPORT_SYMBOL(RTStrFormatNumber);
-
-
-
-/**
- * Formats an integer number according to the parameters.
- *
- * @returns   Length of the number.
- * @param     psz            Pointer to output string.
- * @param     ullValue       Value. Using the high part is optional.
- * @param     uiBase         Number representation base.
- * @param     cchWidth       Width
- * @param     cchPrecision   Precision.
- * @param     fFlags         Flags (NTFS_*).
- */
-static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, signed int cchWidth, signed int cchPrecision,
-                             unsigned int fFlags)
 {
     const char     *pachDigits = "0123456789abcdef";
     char           *pszStart = psz;
     int             cchMax;
     int             cchValue;
-    unsigned long   ul;
     int             i;
     int             j;
+    char            chSign;
 
     /*
      * Validate and adjust input...
      */
-    Assert(uiBase >= 2 || uiBase <= 16);
+    Assert(uiBase >= 2 && uiBase <= 16);
     if (fFlags & RTSTR_F_CAPITAL)
         pachDigits = "0123456789ABCDEF";
     if (fFlags & RTSTR_F_LEFT)
@@ -192,14 +147,25 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
         fFlags &= ~RTSTR_F_THOUSAND_SEP;
 
     /*
-     * Determine value length
+     * Determine value length and sign.  Converts the u64Value to unsigned.
      */
     cchValue = 0;
-    if (ullValue.ulHi || (fFlags & RTSTR_F_64BIT))
+    chSign = '\0';
+    if ((fFlags & RTSTR_F_64BIT) || (u64Value & UINT64_C(0xffffffff00000000)))
     {
-        uint64_t    u64 = *(uint64_t *)(void *)&ullValue;
-        if ((fFlags & RTSTR_F_VALSIGNED) && (ullValue.ulHi & 0x80000000))
-            u64 = -(int64_t)u64;
+        uint64_t u64;
+        if (!(fFlags & RTSTR_F_VALSIGNED) || !(u64Value & RT_BIT_64(63)))
+            u64 = u64Value;
+        else if (u64Value != RT_BIT_64(63))
+        {
+            chSign = '-';
+            u64 = u64Value = -(int64_t)u64Value;
+        }
+        else
+        {
+            chSign = '-';
+            u64 = u64Value = RT_BIT_64(63);
+        }
         do
         {
             cchValue++;
@@ -208,12 +174,24 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
     }
     else
     {
-        ul = (fFlags & RTSTR_F_VALSIGNED) && (ullValue.ulLo & 0x80000000) ? -(int32_t)ullValue.ulLo : ullValue.ulLo;
+        uint32_t u32 = (uint32_t)u64Value;
+        if (!(fFlags & RTSTR_F_VALSIGNED) || !(u32 & UINT32_C(0x80000000)))
+        { /* likley */ }
+        else if (u32 != UINT32_C(0x80000000))
+        {
+            chSign = '-';
+            u64Value = u32 = -(int32_t)u32;
+        }
+        else
+        {
+            chSign = '-';
+            u64Value = u32 = UINT32_C(0x80000000);
+        }
         do
         {
             cchValue++;
-            ul /= uiBase;
-        } while (ul);
+            u32 /= uiBase;
+        } while (u32);
     }
     if (fFlags & RTSTR_F_THOUSAND_SEP)
     {
@@ -229,13 +207,8 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
     i = 0;
     if (fFlags & RTSTR_F_VALSIGNED)
     {
-        if ((ullValue.ulHi || (fFlags & RTSTR_F_64BIT) ? ullValue.ulHi : ullValue.ulLo) & 0x80000000)
-        {
-            ullValue.ulLo = -(int32_t)ullValue.ulLo;
-            if (ullValue.ulHi)
-                ullValue.ulHi = ~ullValue.ulHi;
-            psz[i++] = '-';
-        }
+        if (chSign != '\0')
+            psz[i++] = chSign;
         else if (fFlags & (RTSTR_F_PLUS | RTSTR_F_BLANK))
             psz[i++] = (char)(fFlags & RTSTR_F_PLUS ? '+' : ' ');
     }
@@ -288,9 +261,9 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
      */
     psz += cchValue;
     i = -1;
-    if (ullValue.ulHi || (fFlags & RTSTR_F_64BIT))
+    if ((fFlags & RTSTR_F_64BIT) || (u64Value & UINT64_C(0xffffffff00000000)))
     {
-        uint64_t    u64 = *(uint64_t *)(void *)&ullValue;
+        uint64_t u64 = u64Value;
         if (fFlags & RTSTR_F_THOUSAND_SEP)
         {
             do
@@ -312,24 +285,24 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
     }
     else
     {
-        ul = (fFlags & RTSTR_F_VALSIGNED) && (ullValue.ulLo & 0x80000000) ? -(int32_t)ullValue.ulLo : ullValue.ulLo;
+        uint32_t u32 = (uint32_t)u64Value;
         if (fFlags & RTSTR_F_THOUSAND_SEP)
         {
             do
             {
                 if ((-i - 1) % 4 == 3)
                     psz[i--] = ' ';
-                psz[i--] = pachDigits[ul % uiBase];
-                ul /= uiBase;
-            } while (ul);
+                psz[i--] = pachDigits[u32 % uiBase];
+                u32 /= uiBase;
+            } while (u32);
         }
         else
         {
             do
             {
-                psz[i--] = pachDigits[ul % uiBase];
-                ul /= uiBase;
-            } while (ul);
+                psz[i--] = pachDigits[u32 % uiBase];
+                u32 /= uiBase;
+            } while (u32);
         }
     }
 
@@ -343,6 +316,7 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
     *psz = '\0';
     return (unsigned)(psz - pszStart);
 }
+RT_EXPORT_SYMBOL(RTStrFormatNumber);
 
 
 /**
@@ -363,8 +337,9 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
 RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRFORMAT pfnFormat, void *pvArgFormat,
                             const char *pszFormat, va_list InArgs)
 {
+    char        szTmp[64]; /* Worker functions assumes 64 byte buffer! Ugly but faster. */
     va_list     args;
-    KSIZE       cch = 0;
+    size_t      cch = 0;
     const char *pszStartOutput = pszFormat;
 
     va_copy(args, InArgs); /* make a copy so we can reference it (AMD64 / gcc). */
@@ -406,9 +381,9 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                 }
 
                 /* width */
-                if (ISDIGIT(*pszFormat))
+                if (RT_C_IS_DIGIT(*pszFormat))
                 {
-                    for (cchWidth = 0; ISDIGIT(*pszFormat); pszFormat++)
+                    for (cchWidth = 0; RT_C_IS_DIGIT(*pszFormat); pszFormat++)
                     {
                         cchWidth *= 10;
                         cchWidth += *pszFormat - '0';
@@ -431,9 +406,9 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                 if (*pszFormat == '.')
                 {
                     pszFormat++;
-                    if (ISDIGIT(*pszFormat))
+                    if (RT_C_IS_DIGIT(*pszFormat))
                     {
-                        for (cchPrecision = 0; ISDIGIT(*pszFormat); pszFormat++)
+                        for (cchPrecision = 0; RT_C_IS_DIGIT(*pszFormat); pszFormat++)
                         {
                             cchPrecision *= 10;
                             cchPrecision += *pszFormat - '0';
@@ -519,14 +494,13 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                     /* char */
                     case 'c':
                     {
-                        char ch;
-
                         if (!(fFlags & RTSTR_F_LEFT))
                             while (--cchWidth > 0)
                                 cch += pfnOutput(pvArgOutput, " ", 1);
 
-                        ch = (char)va_arg(args, int);
-                        cch += pfnOutput(pvArgOutput, SSToDS(&ch), 1);
+                        szTmp[0] = (char)va_arg(args, int);
+                        szTmp[1] = '\0';                     /* Some output functions wants terminated strings. */
+                        cch += pfnOutput(pvArgOutput, &szTmp[0], 1);
 
                         while (--cchWidth > 0)
                             cch += pfnOutput(pvArgOutput, " ", 1);
@@ -554,13 +528,13 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                             cchWidth -= cchStr;
                             while (cchStr-- > 0)
                             {
-/**@todo #ifndef IN_RC*/
+/** @todo \#ifndef IN_RC*/
 #ifdef IN_RING3
                                 RTUNICP Cp;
                                 RTUtf16GetCpEx(&pwszStr, &Cp);
-                                char szUtf8[8]; /* Cp=0x7fffffff -> 6 bytes. */
-                                char *pszEnd = RTStrPutCp(szUtf8, Cp);
-                                cch += pfnOutput(pvArgOutput, szUtf8, pszEnd - szUtf8);
+                                char *pszEnd = RTStrPutCp(szTmp, Cp);
+                                *pszEnd = '\0';
+                                cch += pfnOutput(pvArgOutput, szTmp, pszEnd - szTmp);
 #else
                                 char ch = (char)*pwszStr++;
                                 cch += pfnOutput(pvArgOutput, &ch, 1);
@@ -588,11 +562,10 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                             cchWidth -= cchStr;
                             while (cchStr-- > 0)
                             {
-/**@todo #ifndef IN_RC*/
+/** @todo \#ifndef IN_RC*/
 #ifdef IN_RING3
-                                char szUtf8[8]; /* Cp=0x7fffffff -> 6 bytes. */
-                                char *pszEnd = RTStrPutCp(szUtf8, *puszStr++);
-                                cch += pfnOutput(pvArgOutput, szUtf8, pszEnd - szUtf8);
+                                char *pszEnd = RTStrPutCp(szTmp, *puszStr++);
+                                cch += pfnOutput(pvArgOutput, szTmp, pszEnd - szTmp);
 #else
                                 char ch = (char)*puszStr++;
                                 cch += pfnOutput(pvArgOutput, &ch, 1);
@@ -632,7 +605,6 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                     case 'x':
                     case 'X':
                     {
-                        char        achNum[64]; /* FIXME */
                         int         cchNum;
                         uint64_t    u64Value;
 
@@ -660,6 +632,7 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
 
                             case 'X':
                                 fFlags |= RTSTR_F_CAPITAL;
+                                RT_FALL_THRU();
                             case 'x':
                                 uBase = 16;
                                 break;
@@ -753,8 +726,8 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                                 fFlags |= RTSTR_GET_BIT_FLAG(unsigned int);
                             }
                         }
-                        cchNum = RTStrFormatNumber((char *)SSToDS(&achNum), u64Value, uBase, cchWidth, cchPrecision, fFlags);
-                        cch += pfnOutput(pvArgOutput, (char *)SSToDS(&achNum), cchNum);
+                        cchNum = RTStrFormatNumber((char *)&szTmp, u64Value, uBase, cchWidth, cchPrecision, fFlags);
+                        cch += pfnOutput(pvArgOutput, (char *)&szTmp, cchNum);
                         break;
                     }
 
