@@ -1,10 +1,10 @@
-/* $Id: time.h 123408 2018-07-04 10:13:40Z bird $ */
+/* $Id: time.h 135976 2020-02-04 10:35:17Z bird $ */
 /** @file
  * IPRT - Internal RTTime header
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,8 +24,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___internal_time_h
-#define ___internal_time_h
+#ifndef IPRT_INCLUDED_INTERNAL_time_h
+#define IPRT_INCLUDED_INTERNAL_time_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/types.h>
 
@@ -39,4 +42,4 @@ extern DECLHIDDEN(uint64_t) g_u64ProgramStartNanoTS;
 
 RT_C_DECLS_END
 
-#endif
+#endif /* !IPRT_INCLUDED_INTERNAL_time_h */
